﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.UI;
using System.Web.UI.MobileControls;
using System.Web.UI.WebControls;

namespace CheckBoxPersistGridView
{
    public partial class Default : System.Web.UI.Page
    {
        public const string SELECTED_CUSTOMERS_INDEX = "SelectedCustomersIndex"; 

        protected void Page_Load(object sender, EventArgs e)
        {
            if(!Page.IsPostBack)
            {
                BindData();
            }
        }

        private void BindData()
        {
            using(var db = new NorthwindDataContext())
            {
                gvCustomers.DataSource = from c in db.Customers
                                         select c; 
                gvCustomers.DataBind();
            }

            RePopulateCheckBoxes(); 
        }

        private void RePopulateCheckBoxes()
        {
            foreach (GridViewRow row in gvCustomers.Rows)
            {
                var chkBox = row.FindControl("chkSelect") as CheckBox;

                IDataItemContainer container = (IDataItemContainer)chkBox.NamingContainer;

                if (SelectedCustomersIndex != null)
                {
                    if (SelectedCustomersIndex.Exists(i => i == container.DataItemIndex))
                    {
                        chkBox.Checked = true;
                    }
                }
            }
        }

        protected void gvCustomersPageChanging(object sender, GridViewPageEventArgs e)
        {
            foreach(GridViewRow row in gvCustomers.Rows)
            {
                var chkBox = row.FindControl("chkSelect") as CheckBox;

                IDataItemContainer container = (IDataItemContainer) chkBox.NamingContainer; 

                if(chkBox.Checked)
                {
                    PersistRowIndex(container.DataItemIndex); 
                }
                else
                {
                    RemoveRowIndex(container.DataItemIndex);
                }
            }

            gvCustomers.PageIndex = e.NewPageIndex;
            BindData(); 
        }

        private List<Int32> SelectedCustomersIndex
        {
            get
            {
                if(ViewState[SELECTED_CUSTOMERS_INDEX] == null)
                {
                    ViewState[SELECTED_CUSTOMERS_INDEX] = new List<Int32>(); 
                }

                return (List<Int32>) ViewState[SELECTED_CUSTOMERS_INDEX]; 
            }
        }

        private void RemoveRowIndex(int index)
        {
            SelectedCustomersIndex.Remove(index); 
        }

        private void PersistRowIndex(int index)
        {
            if(!SelectedCustomersIndex.Exists(i => i == index))
            {
                SelectedCustomersIndex.Add(index);
            }
        }
    }
}
